"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Drivers_1 = __importDefault(require("./Drivers"));
const Midi_1 = __importDefault(require("./Service/Midi"));
class LaunchpadCore {
    constructor(driverName) {
        this.callbacks = {
            onMidiIn: [],
            onEnabled: [],
            onDisabled: [],
        };
        if (!(driverName in LaunchpadCore.drivers))
            throw new Error('LaunchpadCore: driver not found.');
        this._driver = LaunchpadCore.drivers[driverName];
        this._instance = new Midi_1.default(this._driver.MidiIn, this._driver.MidiOut);
        this.onEnabled();
        process.on('SIGINT', () => this.onDisabled());
        process.on('EXIT', () => this.onDisabled());
    }
    onEnabled() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._instance.in.connect((data) => {
                this.handleEvent('onMidiIn', data);
            });
            yield this.handleEvent('onEnabled', this._instance, this._driver);
        });
    }
    onDisabled() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.handleEvent('onDisabled', this._instance, this._driver);
            this._instance.closeAll();
        });
    }
    /**
     * Events
     */
    handleEvent(event, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const f of this.callbacks[event])
                f(...args);
        });
    }
    on(event, callback) {
        if (!this.callbacks[event])
            throw new Error(`Unknown event name: '${event}'`);
        this.callbacks[event].push(callback);
    }
}
LaunchpadCore.drivers = Drivers_1.default;
exports.default = LaunchpadCore;
